use std::sync::Arc;

use anyhow::Result;
use crawl::{
    wgpu,
    winit::{
        self,
        dpi::PhysicalSize,
        event::{Event, WindowEvent},
        event_loop::EventLoop,
    },
};

pub struct App<'window> {
    title: String,
    event_loop: EventLoop<()>,
    render_ctx: crawl::Context<'window>,
}

impl<'window> App<'window> {
    pub async fn new(width: u32, height: u32, title: &str) -> Result<Self> {
        log::info!("Initialising window...");
        let size = PhysicalSize::new(width, height);
        let event_loop = EventLoop::new()?;
        let window = Arc::new(
            winit::window::WindowBuilder::new()
                .with_title(title)
                .with_inner_size(size)
                .build(&event_loop)?,
        );

        let render_ctx = crawl::Context::new(
            window,
            wgpu::Limits {
                max_storage_buffer_binding_size: 1 << 30,
                max_buffer_size: 1 << 30,
                ..Default::default()
            },
        )
        .await?;

        Ok(Self {
            title: title.to_owned(),
            event_loop,
            render_ctx,
        })
    }

    pub fn run(mut self) -> Result<()> {
        self.event_loop.run(|event, elwt| match event {
            Event::WindowEvent { window_id, event } if window_id == self.render_ctx.window.id() => {
                if self.render_ctx.handle_window_event(&event, elwt) {
                    return;
                }

                if let WindowEvent::RedrawRequested = event {
                    self.render_ctx.window.request_redraw();
                }
            }
            _ => (),
        })?;

        Ok(())
    }
}
